/***********************************************************************************
* Introduction to SQL 
* Dave Valentine
* Advanced Functions Demo Scripts
***********************************************************************************/

USE AdventureWorks2012;
GO

/***********************************************************************************
* ISNULL and COALESCE
*
* This query will return the product sales details from the Sales.SalesOrderDetail 
* table and Production.Product table.  Notice the NULL dates
***********************************************************************************/
SELECT DISTINCT
	  p.productid
	, p.Name
	, DiscontinuedDate
	, SellEndDate
	, SellStartDate
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid



/***********************************************************************************
* This query will return the product sales details from the Sales.SalesOrderDetail 
* table and Production.Product table, if the DiscontinuedDate is NULL, then the
* SellEndDate will be returned in its place.  Notice there are still NULL dates.
***********************************************************************************/
SELECT DISTINCT
	  p.productid
	, p.Name
	, DiscontinuedDate = ISNULL(DiscontinuedDate, SellEndDate)	
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid




/***********************************************************************************
* This query will return the product sales details from the Sales.SalesOrderDetail 
* table and Production.Product table, if the DiscontinuedDate is NULL, then the
* SellEndDate will be returned in its place, if the SellEndDate is NULL, then the
* SellStartDate will be used by adding a year to the time.  Notice the NULL dates.
***********************************************************************************/
SELECT DISTINCT
	  p.productid
	, p.Name
	, DiscontinuedDate = COALESCE(DiscontinuedDate, SellEndDate, DATEADD(YEAR,1,SellStartDate))	
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid





/***********************************************************************************
* CAST and CONVERT
*
* In the String Functions demo scripts we were trying to concatinate Color and 
* SafetyStockLevel from the Production.Product table using the + operator.  We 
* threw an error trying to convert a string value.
***********************************************************************************/
SELECT Name + ' ' + SafetyStockLevel
FROM Production.Product
WHERE ProductID IN (317,318,319,320,321,322)

/***********************************************************************************
* In this updated query we are implmenting a CAST function to change the 
* SafetyStockLevel to a string value.
***********************************************************************************/
SELECT Name + ' ' + CAST(SafetyStockLevel AS NVARCHAR(100))
FROM Production.Product
WHERE ProductID IN (317,318,319,320,321,322)


/***********************************************************************************
* The CONVERT function can be used to convert the SafetyStockLevel to a string value.
* Notice the difference in the function call parameters.
***********************************************************************************/
SELECT Name + ' ' + CONVERT(NVARCHAR(100), SafetyStockLevel)
FROM Production.Product
WHERE ProductID IN (317,318,319,320,321,322)


/***********************************************************************************
* The CONVERT function can also be used to convert dates to string values, and 
* provide formatting to many date standards.  See the list of date formats available
* in the following URL
* https://docs.microsoft.com/en-us/sql/t-sql/functions/cast-and-convert-transact-sql

* This query is converting the SellStartDate column to a string value and formats
* the date string with four different formats.
***********************************************************************************/
SELECT DISTINCT
	  p.productid
	, p.Name
	, SellStartDate = CONVERT(NVARCHAR(20),SellStartDate,102)
	, SellStartDate = CONVERT(NVARCHAR(20),SellStartDate,106)
	, SellStartDate = CONVERT(NVARCHAR(20),SellStartDate,112)
	, SellStartDate = CONVERT(NVARCHAR(20),SellStartDate,126)
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid


/***********************************************************************************
* End Advanced Functions Demo Scripts
***********************************************************************************/

